@@ -9,7 +9,7 @@ use Config::JFDI;
 use Carp qw/croak/;
 use Data::Dumper;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 subtype MyConf => as 'HashRef';
 coerce 'MyConf'
@@ -166,27 +166,12 @@ sub _build_tree {
 
   my $tree = $self->tree;
   foreach my $id ( sort keys %$tree ) {
-    my $node    = $tree->{$id};
-    my $parent  = $tree->{$node->{parent}};
-    $node->{id} = $id;
+      my $node        = $tree->{$id};
+      $node->{id}     = $id;
+      my $parent      = $tree->{$node->{parent}};
+      $node->{parent} = $parent;
 
-    if ( ! $parent ){
-      $self->root($node);
-      next;
-    }
-
-    my $parent_type = $parent->{$self->_type_field} ||'';
-    my $node_type   = $node->{$self->_type_field}   ||'';
-
-    my $key_children;
-    if ( "$node_type" eq "$parent_type") {
-            $key_children = 'children' }
-    else {  $key_children = 'children_' . $node_type }
-
-    my $children_parent = defined $parent->{$key_children} ? $parent->{$key_children} : [];
-    push(@$children_parent, $node );
-    $parent->{$key_children} = $children_parent;
-    $node->{parent}     = $parent;
+      $self->add($parent, $node);
   }
 }
 
@@ -251,6 +236,7 @@ sub _update_children_position {
     }
 }
 
+
 # removes the child's father
 sub _remove_child_father{
     my $self = shift;
@@ -462,36 +448,92 @@ sub del {
 sub add {
   my ($self, $parent, $node, $position) = @_;
 
-  $node->{parent} = $parent;
-  $node->{id}     = $self->_last_node->{id} +1;
+  $node->{id}     = $self->_last_node->{id} +1
+      if ! defined $node->{id};
 
-  my $update_children = 0;
-  if ( $parent ) {
-      if ( $position ) {
-          splice @{$parent->{children}}, $position -1, 0, $node;
-          $update_children = 1;
-      }
-      else {
-          push(@{$parent->{children}}, $node);
-          if ( defined ${$parent->{children}}[0] && ${$parent->{children}}[0]->{$self->_position_field}) {
-              $update_children = 1;
-          }
-      }
-  }
-  else {
+
+  if ( ! $parent ){
       if ( $self->root) {
           die "root already exist !";
       }
       $self->root($node);
   }
+
+  $node->{parent} = $parent;
+
+  my $is_exist_in_backend;
+  $is_exist_in_backend = 1
+      if $self->tree->{$node->{id}};
+
+  # save node in tree
+  $self->tree->{$node->{id}} = $node;
+
+
+  my $key_children;
+  # node == root
+  if ( ! defined $parent || ! $parent ) {
+      return $self->root;
+  }
+  else {
+      #if parent have a type
+      if ( defined $parent->{$self->_type_field}) {
+
+          # if node have a type
+          if ( defined $node->{$self->_type_field}) {
+              #$self->check_rules($node, $parent);
+
+              # if node and parent have the same type
+              if ( $node->{$self->_type_field} eq $parent->{$self->_type_field} ) {
+                  $key_children = 'children';
+              }
+              else {
+                  $key_children = 'children_' . $node->{$self->_type_field};
+              }
+          }
+          # else node have the same type as parent
+          else {
+              $node->{$self->_type_field} = $parent->{$self->_type_field};
+              $key_children = 'children';
+          }
+      }
+      # parent haven't type
+      else {
+          if ( defined $node->{$self->_type_field}) {
+              die 'node have a type but not the parent !';
+          }
+          else {
+              $key_children = 'children';
+          }
+      }
+
+  }
+
+
+
+  my $update_children = 0;
+  if ( $parent ) {
+      if ( $position ) {
+          splice @{$parent->{$key_children}}, $position -1, 0, $self->tree->{$node->{id}};
+          $update_children = 1;
+      }
+      else {
+          push(@{$parent->{$key_children}}, $self->tree->{$node->{id}});
+          if ( defined ${$parent->{$key_children}}[0] && ${$parent->{$key_children}}[0]->{$self->_position_field}) {
+              $update_children = 1;
+          }
+      }
+  }
+
+
+
   # add node in tree
-  $self->_bckd_create($node, 'add node ' . $node->{$self->_search_field});
+  $self->_bckd_create($node, 'add node ' . $node->{$self->_search_field})
+      if ! $is_exist_in_backend;
 
   # recalc position of children
   $self->_update_children_position(@{$parent->{children}}) if $update_children;
 
-  # save node in tree
-  $self->tree->{$node->{id}} = $node;
+  return $node;
 }
 
 sub insert_before {
@@ -652,6 +694,15 @@ TreePath - Simple Tree Path!
 
     => 'parent' is a reference on root node and 'children' is an array containing 2 nodes
 
+    => if nodes have 'type', it's possible to 'link' some 'type' of node on some others types
+
+       For example, A:(type1), B:(type1, parent:A), C:(type2, parent:A) becomes :
+       A:(type1, children:[B], children_type2[C])
+
+       Each children can be ordered : { name => A, position => 2};
+
+       It's also possible to block link with 'rules'. 'link' must be follow these rules to be applied.
+
 =cut
 
 =head2 search (hashref)
